<?php

namespace Bydedi\Http\EventListener;

use Bydedi\Http\Event\MatchRequestEvent;
use Bydedi\Http\Exception\MethodNotAllowedHttpException;
use Bydedi\Http\Exception\RedirectHttpException;
use Bydedi\Http\HttpEvents;
use Bydedi\Http\Exception\NotFoundHttpException;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\Routing\Exception\MethodNotAllowedException;
use Symfony\Component\Routing\Exception\ResourceNotFoundException;
use Symfony\Component\Routing\Matcher\RequestMatcherInterface;

class RouterListener implements EventSubscriberInterface
{

    /**
     * @var \Symfony\Component\Routing\Matcher\RequestMatcherInterface
     */
    protected $matcher;
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    public function __construct(RequestMatcherInterface $matcher, LoggerInterface $logger = null)
    {
        $this->matcher = $matcher;
        $this->logger = $logger;
    }

    public function onRequest(MatchRequestEvent $event)
    {

        $request = $event->getRequest();

        if ($request->attributes->has('_controller')) {
            // routing is already done
            return;
        }

        $this->matcher->getContext()->fromRequest($request);

        if (null !== $this->logger) {
            // TODO : example below
            // $this->logger->info(sprintf('Matched route "%s" (parameters: %s)', $parameters['_route'], $this->parametersToString($parameters)));
        }

        try {
            // on match la request avec une url
            $request->attributes->add($this->matcher->matchRequest($request));
        } catch (RedirectHttpException $e) {
            $event->setResponse(new RedirectResponse($e->getUrl()));
        } catch (ResourceNotFoundException $e) {
            $message = sprintf('No route found for "%s %s"', $request->getMethod(), $request->getPathInfo());

            if ($referer = $request->headers->get('referer')) {
                $message .= sprintf(' (from "%s")', $referer);
            }

            throw new NotFoundHttpException($message, $e);

        } catch (MethodNotAllowedException $e) {
            $message = sprintf('No route found for "%s %s": Method Not Allowed (Allow: %s)', $request->getMethod(), $request->getPathInfo(), implode(', ', $e->getAllowedMethods()));

            throw new MethodNotAllowedHttpException($e->getAllowedMethods(), $message, $e);
        }
    }

    /**
     * @return array
     */
    public static function getSubscribedEvents()
    {
        return [
            HttpEvents::REQUEST => array(array('onRequest', 32)),
        ];
    }
}
